unit LABDATE;
     (* ----------- > DELPHI 2.0 only < --------------
        This NON-visual component allows you to easily work with dates.
        It uses the standard Delphi date routines, and is merely
        encapsulating those routines.   It does allow a few extra things,
        such as:
             Incrementing/decrementing by a given number of
             months, days, or years (or all 3) !   It returns the Julian
             date, an alpha version of the Day-of-the-week, and an alpha
             version of the month.

        This unit has only 3 methods you can invoke:
             1) UseCurrDate - which sets the BASE dates to the
                current date of your PC.  (default)
             2) ChangeDate - which causes the dates to CHANGE by
                the values specified in the fields IncDay, IncMonth,
                and IncYear.  (positive or negative values)
             3) UseNewDate - which causes the values in the fields
                NewYY, NewDD, NewMM to become the new base date.

        Created by Larry A. Brodahl Saturday, July 6, 1996

        This is free!  Can be used by anyone for anything!  I would
        appreciate you keeping my name in the code if you change
        anything though.
     *)

interface

uses
  Windows, Messages, SysUtils, Classes;

type
  TLABDATE = class(TComponent)
  private
    { Private declarations }
    FBaseDay    : Word;
    FBaseMonth  : Word;
    FBaseYear   : Word;
    FIncDay    : Integer;
    FIncMonth  : Integer;
    FIncYear   : Integer;
    FNewDay    : Word;
    FNewMonth  : Word;
    FNewYear   : Word;
    FStrDayOfWeek : String;
    FStrMonth     : String;
    FJulianDate   : integer;
    FEncodedDate  : TDateTime;
    Procedure SetBaseDay(Value : Word);
    Function GetBaseDay : Word;
    Procedure SetBaseMonth(Value : Word);
    Function GetBaseMonth : Word;
    Procedure SetBaseYear(Value : Word);
    Function GetBaseYear : Word;
       (* fields that hold increment values *)
    Procedure SetIncDay(Value : Integer);
    Function GetIncDay : Integer;
    Procedure SetIncMonth(Value : Integer);
    Function GetIncMonth : Integer;
    Procedure SetIncYear(Value : Integer);
    Function GetIncYear : Integer;
       (* fields that hold NEW date. IE: date to set as base *)
    Procedure SetNewDay(Value : Word);
    Function GetNewDay : Word;
    Procedure SetNewMonth(Value : Word);
    Function GetNewMonth : Word;
    Procedure SetNewYear(Value : Word);
    Function GetNewYear : Word;
        (* Misc fields *)
    Function GetJulianDate : integer;
    Function GetStrMonth   : String;
    Function GetStrDayOfWeek : string;
    Function GetEncodedDate : TDateTime;
    Procedure Convert_dates;
    procedure Find_First_dayOfYear(inYY : Word);
    Function Calc_Julian_date(CurrDate, FirstDay: TdateTime) : integer;
    Procedure Find_Feb_29(InYY : word);
    Procedure SetJulianDate(value : INteger);
    Procedure SetStrMonth(Value : string);
    Procedure SetStrDayOfWeek(value : string);
    Procedure SetEncodedDate(value : TDateTime);

  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor create(Aowner : TComponent); override;
    Destructor Destroy; override;
    Procedure  ChangeDate;
    Procedure  UseCurrDate;
    Procedure  UseNewDate;
  published
    { Published declarations }
       Property BaseDD : word
                Read  GetBaseDay
                Write SetBaseDay;
       Property BaseMM : word
                Read  GetBaseMonth
                Write SetBaseMonth;
       Property BaseYY : word
                Read  GetBaseYear
                Write SetBaseYear;
       Property IncDD : Integer
                Read  GetIncDay
                Write SetIncDay;
       Property IncMM : Integer
                Read  GetIncMonth
                Write SetIncMonth;
       Property IncYY : Integer
                Read  GetIncYear
                Write SetIncYear;
       Property NewDD : word
                Read  GetNewDay
                Write SetNewDay;
       Property NewMM : word
                Read  GetNewMonth
                Write SetNewMonth;
       Property NewYY : word
                Read  GetNewYear
                Write SetNewYear;

       Property JulianDate   : integer
                Read GetJulianDate
                Write SetJulianDate;
       Property StrMonth     : String
                Read  GetStrMonth
                Write SetStrMonth;
       Property StrDayOfWeek : String
                Read  GetStrDayOfWeek
                Write SetStrDayOfWeek;
       Property EncodedDate  : TdateTime
                Read GetEncodedDate
                Write SetEncodedDate;
  end;

procedure Register;

implementation
var
    Gl_currDate  : Tdatetime;  (* Current date - global *)
    Gl_FirstDOY  : TDatetime;  (* Holds 1st day of a given year *)

    gl_MonthArr  : array[1..12] of string =
                 ('January', 'February', 'March', 'April', 'May', 'June',
                  'July', 'August', 'September', 'October', 'November',
                  'December');
    gl_dowArr    : array[1..7] of string =
                   ('Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday',
                    'Friday', 'Saturday');
    Gl_DaysInMonthArr : Array[1..12] of integer =
                   (31,28,31,30,31,30,31,31,30,31,30,31);
Procedure TLABDATE.SetJulianDate(value : integer);
begin
//
end;
Procedure TLABDATE.SetStrMonth(value : string);
begin
//
end;
Procedure TLABDATE.SetStrDayOfWeek(value : string);
begin
//
end;
Procedure TLABDATE.SetEncodedDate(value : TDateTime);
begin
//
end;

function TLABDATE.GetEncodedDate : TDATETIME;
BEGIN
   GetEncodedDate := FEncodedDate;
END;

procedure TLABDATE.Find_First_DayOfYear(inYY : word);
var
  l_YY, l_MM, l_DD : word;
begin
   l_mm := 1;   (* January *)
   l_dd := 1;   (* 1st *)
   l_yy := inYY;
   try
   gl_FirstDoy := EncodeDate(l_yy, l_mm, l_DD);
   except
   end;
end;

Procedure TLABDATE.Find_Feb_29(InYY : word);
var
   l_mm, l_dd : word;
begin
   try
      l_mm := 02;  (* set to Feb 29 *)
      l_dd := 29;
      EncodeDate(InYY, l_mm, l_DD);
      Gl_DaysInMonthArr[2] := 29;  (* Leap Year *)
   except
      Gl_DaysInMonthArr[2] := 28;  (* No Leap year *)
   end;
end;


Function TLABDATE.Calc_Julian_date(CurrDate, FirstDay: TdateTime) : integer;
begin
   Calc_Julian_date := Trunc(((CurrDate - FirstDay) + 1.0));
end;

procedure TLABDATE.Convert_dates;
var
  l_dayOfWeek : Word;
begin
   try
   FEncodedDate := EncodeDate(FBaseYear, FBaseMonth, FBaseDay);
   Find_First_DayOfYear(FBaseYear);  (* Sets global 1st day of year *)
   FJulianDate := Calc_Julian_Date(gl_CurrDate, Gl_FirstDoy);
   l_DayOFWeek := DayOfWeek(gl_CurrDate);
   FStrDayOfWeek := gl_DowArr[l_DayOfWeek];  (* cvrt to string *)
   FStrMonth := gl_monthArr[FBaseMonth];        (* Cvrt to string *)
      (* Set Feb number of days to 29 if Feb and LeapYear *)
   Find_Feb_29(FBaseYear);
   except
      raise;
   end;
end;

Constructor TLABDATE.create(Aowner : TComponent);
begin
   inherited create(AOwner);
   gl_currdate := Date;       (* Get PC date *)
   DecodeDate(Gl_CurrDate, FBaseYear, FBaseMonth, FBaseDay); (* cvrt date *)
   Convert_dates;    (* put current date in hold area *)
      (* Set default values *)
   FIncDay := 0;
   FIncMonth := 0;
   FIncYear := 0;
   FNewDay := 0;
   FNewMonth := 0;
   FNewYear := 0;
end;

Destructor TLABDATE.Destroy;
begin
   inherited Destroy;
end;


Procedure  TLABDATE.ChangeDate;
var
   l_ktr : integer;
   l_addAmt : integer;
   l_end    : integer;
begin
   FBaseYear := FBaseYear + FIncYear;
   try
   gl_currDate := EncodeDate(FBaseYear, FBaseMonth, FBaseDay); (* cvrt date *)
   except
   end;
     (* Calc Months *)
   if FIncMonth < 0 then
      begin
         l_end := FIncMonth * -1;
         l_addAmt := -1;
      end
      else
      begin
         l_end := FincMonth;
         l_addAmt := 1;
      end;
   for l_ktr := 1 to l_end do
      begin
         FBaseMonth := FBaseMonth + L_addAmt;
         if FBaseMonth > 12 then
            begin
              Inc(FBaseYear);
              FBaseMonth := 1;
            end;
         if FBaseMonth < 1 then
            begin
               Dec(FBaseYear);
               FBaseMonth := 1;
            end;
      end;  (* loop *)
   try
   gl_currDate := EncodeDate(FBaseYear, FBaseMonth, FBaseDay); (* cvrt date *)
   except
   end;

      (* Calc days *)
   GL_CURRDATE := GL_CURRDATE + FincDay;
   try
      DecodeDate(Gl_CurrDate, FBaseYear, FBaseMonth, FBaseDay); (* cvrt date *)
      Convert_dates;
   finally
   end;
end;





Procedure  TLABDATE.UseCurrDate;
begin
   gl_currdate := Date;       (* Get PC date *)
   try
   DecodeDate(Gl_CurrDate, FBaseYear, FBaseMonth, FBaseDay); (* cvrt date *)
   Convert_dates;
   except
   end;
end;

Procedure  TLABDATE.UseNewDate;
var
  l_date : TdateTime;
begin
   try
   l_date := EncodeDate(FNewYear, FNewMonth, FNewDay); (* cvrt date *)
   gl_currDate := l_date;
   FBaseDay := FNewDay;
   FBaseMonth := FNewMonth;
   FBaseYear  := FNewYear;
   Convert_dates;
   except
      raise;
   end;
end;




Procedure TLABDATE.SetBaseDay(Value : Word);
begin
   FBaseDay := value;
end;

Function TLABDATE.GetBaseDay : Word;
begin
   GetBaseDay := FBaseDay;
end;
Procedure TLABDATE.SetBaseMonth(Value : Word);
begin
   FBaseMonth := value;
end;
Function TLABDATE.GetBaseMonth : Word;
begin
   GetBaseMonth := FBaseMonth;
end;
Procedure TLABDATE.SetBaseYear(Value : Word);
begin
   FBaseYear := Value;
end;
Function TLABDATE.GetBaseYear : Word;
begin
  GetBaseYear := FBaseYear;
end;
Procedure TLABDATE.SetIncDay(Value : Integer);
begin
   FIncDay := Value;
end;
Function TLABDATE.GetIncDay : Integer;
begin
   GetIncDay := FIncDay;
end;
Procedure TLABDATE.SetIncMonth(Value : Integer);
begin
   FIncMonth := Value;
end;
Function TLABDATE.GetIncMonth : Integer;
begin
   GetIncMonth := FincMonth;
end;
Procedure TLABDATE.SetIncYear(Value : Integer);
begin
   FincYear := value;
end;
Function TLABDATE.GetIncYear : Integer;
begin
   GetIncYear := FincYear;
end;


Procedure TLABDATE.SetNewDay(Value : Word);
begin
   FNewDay := Value;
end;
Function TLABDATE.GetNewDay : Word;
begin
   GetNewDay := FNewDay;
end;
Procedure TLABDATE.SetNewMonth(Value : Word);
begin
   FNewMonth := Value;
end;
Function TLABDATE.GetNewMonth : Word;
begin
   GetNewMonth := FNewMonth;
end;
Procedure TLABDATE.SetNewYear(Value : Word);
begin
   FNewYear := value;
end;
Function TLABDATE.GetNewYear : Word;
begin
   GetNewYear := FNewYear;
end;


Function TLABDATE.GetJulianDate : integer;
begin
   GetJulianDate := FJulianDate;
end;

Function TLABDATE.GetStrMonth   : String;
begin
   GetStrMonth := FStrMonth;
end;

Function TLABDATE.GetStrDayOfWeek : string;
begin
   GetStrDayOfWeek := FStrDayOfWeek;
end;







procedure Register;
begin
  RegisterComponents('MyStuff', [TLABDATE]);
end;

end.
